@@ -1,12 +1,19 @@
-# $Id: Ping.pm,v 1.3 2001/10/13 11:08:50 rcaputo Exp $
+# $Id: Ping.pm,v 1.4 2001/10/17 16:21:49 rcaputo Exp $
 # License and documentation are after __END__.
 
 package POE::Component::Client::Ping;
 
+use Exporter;
+@ISA = qw(Exporter);
+@EXPORT_OK = qw(REQ_ADDRESS REQ_TIMEOUT REQ_TIME REQ_USER_ARGS
+		RES_ADDRESS RES_ROUNDTRIP RES_TIME);
+%EXPORT_TAGS = ('const' => [ qw(REQ_ADDRESS REQ_TIMEOUT REQ_TIME REQ_USER_ARGS
+				RES_ADDRESS RES_ROUNDTRIP RES_TIME) ]
+		);
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '0.94';
+$VERSION = '0.96';
 
 use Carp qw(croak);
 use Symbol qw(gensym);
@@ -25,17 +32,22 @@ sub DEBUG_SOCKET () { 0 }; # Watch the socket open and close.
 sub spawn {
   my $type = shift;
 
-  croak "$type requires root privilege" if $> and ($^O ne 'VMS');
   croak "$type requires an even number of parameters" if @_ % 2;
-
   my %params = @_;
 
+  croak "$type requires root privilege"
+    if $> and ($^O ne 'VMS') and not defined $params{Socket};
+
   my $alias = delete $params{Alias};
   $alias = 'pinger' unless defined $alias and length $alias;
 
   my $timeout = delete $params{Timeout};
   $timeout = 1 unless defined $timeout and $timeout >= 0;
 
+  my $onereply = delete $params{OneReply};
+
+  my $socket = delete $params{Socket};
+
   croak( "$type doesn't know these parameters: ",
          join(', ', sort keys %params)
        ) if scalar keys %params;
@@ -48,7 +60,7 @@ sub spawn {
         got_pong => \&poco_ping_pong,
         _default => \&poco_ping_default,
       },
-      args => [ $alias, $timeout ],
+      args => [ $alias, $timeout, $socket, $onereply ],
     );
 
   undef;
@@ -61,6 +73,17 @@ sub PBS_SESSION      () { 1 };
 sub PBS_ADDRESS      () { 2 };
 sub PBS_REQUEST_TIME () { 3 };
 
+# request_packet offsets
+sub REQ_ADDRESS       () { 0 };
+sub REQ_TIMEOUT       () { 1 };
+sub REQ_TIME          () { 2 };
+sub REQ_USER_ARGS     () { 3 };
+
+# response_packet offsets
+sub RES_ADDRESS       () { 0 };
+sub RES_ROUNDTRIP     () { 1 };
+sub RES_TIME          () { 2 };
+
 # "Static" variables which will be shared across multiple instances.
 
 my $pid = $$ & 0xFFFF;
@@ -70,14 +93,16 @@ my $seq = 0;
 # socket which will be used to ping.
 
 sub poco_ping_start {
-  my ($kernel, $heap, $alias, $timeout) = @_[KERNEL, HEAP, ARG0, ARG1];
-
-  $heap->{data}        = 'Use POE!' x 7;        # 56 data bytes
-  $heap->{data_size}   = length($heap->{data});
-  $heap->{timeout}     = $timeout;
-  $heap->{ping_by_seq} = { };  # keyed on sequence number
-  $heap->{addr_to_seq} = { };  # keyed on request address, then sender
+  my ($kernel, $heap, $alias, $timeout, $socket, $onereply) =
+    @_[KERNEL, HEAP, ARG0..ARG3];
 
+  $heap->{data}          = 'Use POE!' x 7;        # 56 data bytes :)
+  $heap->{data_size}     = length($heap->{data});
+  $heap->{timeout}       = $timeout;
+  $heap->{onereply}      = $onereply;
+  $heap->{ping_by_seq}   = { };  # keyed on sequence number
+  $heap->{addr_to_seq}   = { };  # keyed on request address, then sender
+  $heap->{socket_handle} = $socket if defined $socket;
   $kernel->alias_set($alias);
 }
 
@@ -160,18 +185,25 @@ sub poco_ping_ping {
                          ? $address
                          : inet_aton($address)
                        );
+
   my $socket_address = pack_sockaddr_in(ICMP_PORT, $usable_address);
 
   # Send the packet.  Should an error be checked?
   send($heap->{socket_handle}, $msg, ICMP_FLAGS, $socket_address) or die $!;
 
   # Set a timeout based on the sequence number.
-  $kernel->delay( $seq, => $timeout );
+  $kernel->delay( $seq => $timeout );
 
   # Record information about the ping request.
+  my @user_args = ();
+  if (ref($event) eq "ARRAY") {
+      @user_args = @{ $event };
+      $event = shift @user_args;
+  }
+  
   $heap->{ping_by_seq}->{$seq} =
     [ # PBS_POSTBACK
-      $sender->postback($event, $address, $timeout, time()),
+      $sender->postback($event, $address, $timeout, time(), @user_args),
       "$sender",   # PBS_SESSION (stringified to weaken reference)
       $address,    # PBS_ADDRESS
       time()       # PBS_REQUEST_TIME
@@ -284,6 +316,9 @@ sub poco_ping_pong {
   $heap->{ping_by_seq}->{$from_seq}->[PBS_POSTBACK]->
     ( inet_ntoa($from_ip), $trip_time, $now
     );
+
+  # clear the timer
+  $kernel->delay($from_seq) if $heap->{onereply};
 }
 
 # Default's used to catch ping timeouts, which are named after the
@@ -320,14 +355,15 @@ sub poco_ping_default {
       DEBUG_SOCKET and warn "closing the raw icmp socket";
       $kernel->select_read( delete $heap->{socket_handle} );
     }
+
+    return 1;
   }
   else {
     DEBUG and warn "this shouldn't technically be displayed";
-  }
 
-  # Return 1 in case we've caught a signal.  We'll handle them all,
-  # which will keep us alive until all our clients are gone.
-  return 1;
+    # Let unhandled signals pass through so we do not block SIGINT, etc.
+    return 0;
+  }
 }
 
 1;
@@ -345,6 +381,7 @@ POE::Component::Client::Ping - an ICMP ping client component
   POE::Component::Client::Ping->spawn(
     Alias       => 'pingthing',   # defaults to 'pinger'
     Timeout     => 10,            # defaults to 1 second
+    OneReply    => 1             # defaults to disabled
   );
 
   $kernel->post( 'pingthing', # Post the request to the 'pingthing'.
@@ -378,12 +415,24 @@ POE::Component::Client::Ping - an ICMP ping client component
     }
   }
 
+  or
+
+  use POE::Component::Client::Ping ':const';
+
+  # post an array ref as the callback to get data back to you
+  $kernel->post('pinger', 'ping', [ 'pong', @user_data ]);
+
+  # use the REQ_USER_ARGS constant to get to your data
+  sub got_pong {
+      my ($request, $response) = @_;
+      my @req_data = @{$reqest}[REQ_USER_ARGS..$#{$request}];
+  }
+
 =head1 DESCRIPTION
 
 POE::Component::Client::Ping is non-blocking ICMP ping client session.
 It lets several other sessions ping through it in parallel, and it
 lets them continue doing other things while they wait for responses.
-
 Ping client components are not proper objects.  Instead of being
 created, as most objects are, they are "spawned" as separate sessions.
 To avoid confusion (and hopefully not cause other confusion), they
@@ -403,6 +452,15 @@ even knowing) hard references to them.  It is possible to spawn
 several Ping components with different names, but the author can find
 no point in doing this.
 
+=item Socket => $raw_socket
+
+C<Socket> allows for the caller to open an existing raw socket rather
+than letting PoCo::Client::Ping try and open one itself.  This can be
+used to let startup code create the socket while running with enhanced
+privileges, which are dropped.  People who don't want to audit POE for
+security holes will find this useful since they won't need to run POE
+as root.
+
 =item Timeout => $ping_timeout
 
 C<Timeout> specifies the default amount of time a Ping component will
@@ -413,6 +471,17 @@ set the timeout to a fractional number of seconds.
 The default timeout is only used for ping requests that don't include
 their own timeouts.
 
+=item OneReply => 0|1
+
+C<OneReply>, if set, tells the Ping component to clear the timeout upon
+successful ping.  The Ping component will thus only post one reply back
+to the calling module, upon either success or timeout, but not both.
+This is useful if you are checking if one host is reachable or not as
+you will only receive one reply.  By default, this setting is disabled,
+meaning that you will get the ping timeout in addition to any ping 
+replies.  This is useful if you are expecting multiple replies from one
+ping, such as when pinging a subnet.
+ 
 =back
 
 Sessions communicate asynchronously with PoCo::Client::Ping.  They
@@ -486,6 +555,11 @@ real number based on the current system's time() epoch.
 
 =back
 
+If the :const tagset is imported the following constants will be exported:
+
+REQ_ADDRESS, REQ_TIMEOUT, REQ_TIME, REQ_USER_ARGS, RES_ADDRESS, 
+RES_ROUNDTRIP, RES_TIME
+
 =head1 SEE ALSO
 
 This component's ICMP ping code was lifted from Net::Ping, which is an
